import json
from django.views.decorators.http import require_GET
from graphics.models import GraphicsMetric
from common.utils import JSONResponse
from django.utils.dateformat import format
from datetime import date, timedelta


@require_GET
def result_graphics_resolution(requests,
                               benchmark_filter=None,
                               machine_filter=None,
                               resolution_filter=None):

    show_days = date.today() - timedelta(days=60)
    test_names = [
        graphics.name for graphics in GraphicsMetric.objects.filter(
            name__contains=benchmark_filter,
            machine__name__contains=machine_filter,
            test_resolution=resolution_filter,
        ).distinct('name')]

    results = GraphicsMetric.objects.filter(
        test_resolution=resolution_filter,
        ran_at__gte=show_days,
        ).order_by('ran_at')

    return_list = [
        {"key": test_name,
         "values": [
             values for result in results
             if test_name in result.name and
             hasattr(result, 'jenkins_url') for values in [
                 [format(result.ran_at, 'U'),
                  float(result.average),
                  str(result.test_resolution),
                  str(result.maximum),
                  str(result.minimum),
                  str(result.stddev),
                  str(result.jenkins_url)]
             ]
         ]} for test_name in test_names
    ]

    return JSONResponse(json.dumps(return_list))


@require_GET
def latest_graphics_resolution(requests,
                               benchmark_filter=None,
                               machine_filter=None,
                               resolution_filter=None):

    test_list = []
    machine_names = list(set([graphics.machine.name for graphics in
                              GraphicsMetric.objects.filter(
                                  name__contains=benchmark_filter,
                                  machine__name__contains=machine_filter,
                                  test_resolution=resolution_filter,
                              )]))

    test_names = [
        graphics.name for graphics in GraphicsMetric.objects.filter(
            name__contains=benchmark_filter,
            machine__name__contains=machine_filter,
            test_resolution=resolution_filter,
        ).distinct('name')]
    test_names.sort()

    results = GraphicsMetric.objects.filter(
        name__contains=benchmark_filter,
        test_resolution=resolution_filter,
        machine__name__contains=machine_filter,
        ).order_by('ran_at')

    # put data in list dictionary

    test_list = [
        {"key": test_name,
         "values": [
             values for result in results
             if test_name == result.name and
             result.image.build_number != "" for values in [
                 [float(result.image.build_number),
                  float(result.average),
                  str(result.test_resolution),
                  str(result.maximum),
                  str(result.minimum),
                  str(result.stddev),
                  str(result.jenkins_url)]
             ]
         ]} for test_name in test_names
    ]

    #separate tests by machine
    xmir_x_machines_list = []
    for machine in machine_names:
        machine_test_list = [
            test for test in test_list if machine in test['key']]
        if len(machine_test_list) is 2:
            xmir_x_machines_list.append(machine_test_list)

    return_list = []
    for tests in xmir_x_machines_list:
        iso0 = "test0"
        iso1 = "test1"
        count0 = -1
        count1 = -1
        count0_total = len(tests[0]['values']) - (len(tests[0]['values']) * 2)
        count1_total = len(tests[1]['values']) - (len(tests[1]['values']) * 2)
        while iso0 != iso1 and count0 > count0_total and count1 > count1_total:
            iso0 = tests[0]['values'][count0][0]
            iso1 = tests[1]['values'][count1][0]
            if iso0 < iso1:
                count1 -= 1
            if iso0 > iso1:
                count0 -= 1
        if iso0 == iso1:
            return_list.append({'key': tests[0]['key'],
                                'values': [tests[0]['values'][count0]]})
            return_list.append({'key': tests[1]['key'],
                                'values': [tests[1]['values'][count1]]})

    return JSONResponse(json.dumps(return_list))
