# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import json

from common import management
from common.models import JenkinsBuild, JenkinsJob

from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.client import Client
from django.utils.dateformat import format

from graphics.management.commands import jenkins_pull_phoronix
from graphics.models import GraphicsMetric, GraphicsImage, GraphicsMachine
from graphics.util import jenkins_pull

from mock import Mock
from os import path


class TestGraphicsUrls(TestCase):
    """ test the urls in graphics """

    client = Client()

    def setUp(self):
        self.iso = 20130805
        self.min_1 = str(float(1))
        self.min_10 = str(float(11))
        self.avg_5 = float(5)
        self.avg_50 = float(50)
        self.max_10 = str(float(10))
        self.max_100 = str(float(100))
        self.stddev_4 = str(float(4))
        self.stddev_55 = str(float(55))
        self.framework = 'utah'
        self.fake_url_0 = 'http://www.ubuntu.com/fake/1/2/3/4'
        self.fake_url_1 = 'http://www.ubuntu.com/fake/11/22/33/44-xmir'

        self.ran_at = datetime.datetime.now()
        self.image = GraphicsImage.objects.create(
            release='saucy',
            variant='desktop',
            arch='amd64',
            md5='fakemd5',
            build_number=self.iso,
            )

        self.assert_(self.image)

        self.jenkins_job_0 = JenkinsJob.objects.create(
            name='ps-intel-2500-le-openarena',
            url='http:www.ubuntu.com/0',
            )

        self.jenkins_job_0_xmir = JenkinsJob.objects.create(
            name='ps-intel-2500-le-openarena-xmir',
            url='http:www.ubuntu.com/0',
            )

        self.jenkins_job_1 = JenkinsJob.objects.create(
            name='ps-intel-2500-le-gui-toolkits',
            url='http:www.ubuntu.com/1',
            )

        self.jenkins_job_1_xmir = JenkinsJob.objects.create(
            name='ps-intel-2500-le-gui-toolkits-xmir',
            url='http:www.ubuntu.com/1',
            )

        self.jenkins_build_0 = JenkinsBuild.objects.create(
            job=self.jenkins_job_0,
            failed=False,
            build_number='1',
            ran_at=self.ran_at,
            )

        self.jenkins_build_0_xmir = JenkinsBuild.objects.create(
            job=self.jenkins_job_0_xmir,
            failed=False,
            build_number='2',
            ran_at=self.ran_at,
            )

        self.jenkins_build_1 = JenkinsBuild.objects.create(
            job=self.jenkins_job_0,
            failed=False,
            build_number='3',
            ran_at=self.ran_at,
            )

        self.jenkins_build_1_xmir = JenkinsBuild.objects.create(
            job=self.jenkins_job_0_xmir,
            failed=False,
            build_number='4',
            ran_at=self.ran_at,
            )

        self.jenkins_build_2 = JenkinsBuild.objects.create(
            job=self.jenkins_job_1,
            failed=False,
            build_number='5',
            ran_at=self.ran_at,
            )

        self.jenkins_build_2_xmir = JenkinsBuild.objects.create(
            job=self.jenkins_job_1_xmir,
            failed=False,
            build_number='6',
            ran_at=self.ran_at,
            )

        self.jenkins_build_3 = JenkinsBuild.objects.create(
            job=self.jenkins_job_1,
            failed=False,
            build_number='7',
            ran_at=self.ran_at,
            )

        self.jenkins_build_3_xmir = JenkinsBuild.objects.create(
            job=self.jenkins_job_1_xmir,
            failed=False,
            build_number='8',
            ran_at=self.ran_at,
            )

        self.machine = GraphicsMachine.objects.create(
            name='ps-intel-2500-le',
            mac_address='DE:AD:BE:EF:00',
            )

        self.machine_xmir = GraphicsMachine.objects.create(
            name='ps-intel-2500-le-xmir',
            mac_address='DE:AD:BE:EF:01',
            )

        self.metric_0 = GraphicsMetric.objects.create(
            image=self.image,
            machine=self.machine,
            jenkins_build=self.jenkins_build_0,
            name='ps-intel-2500-le-x-openarena',
            ran_at=self.ran_at,
            minimum=self.min_1,
            maximum=self.max_10,
            average=self.avg_5,
            stddev=self.stddev_4,
            test_resolution='800x600',
            framework=self.framework,
            jenkins_url=self.fake_url_0,
            )

        self.metric_0_xmir = GraphicsMetric.objects.create(
            image=self.image,
            machine=self.machine_xmir,
            jenkins_build=self.jenkins_build_0_xmir,
            name='ps-intel-2500-le-xmir-openarena',
            ran_at=self.ran_at,
            minimum=self.min_1,
            maximum=self.max_10,
            average=self.avg_5,
            stddev=self.stddev_4,
            test_resolution='800x600',
            framework=self.framework,
            jenkins_url=self.fake_url_1,
            )

        self.metric_1 = GraphicsMetric.objects.create(
            image=self.image,
            machine=self.machine,
            jenkins_build=self.jenkins_build_1,
            name='ps-intel-2500-le-x-openarena',
            ran_at=self.ran_at,
            minimum=self.min_1,
            maximum=self.max_10,
            average=self.avg_5,
            stddev=self.stddev_4,
            test_resolution='800x600',
            framework=self.framework,
            jenkins_url=self.fake_url_0,
            )

        self.metric_1_xmir = GraphicsMetric.objects.create(
            image=self.image,
            machine=self.machine_xmir,
            jenkins_build=self.jenkins_build_1_xmir,
            name='ps-intel-2500-le-xmir-openarena',
            ran_at=self.ran_at,
            minimum=self.min_1,
            maximum=self.max_10,
            average=self.avg_5,
            stddev=self.stddev_4,
            test_resolution='800x600',
            framework=self.framework,
            jenkins_url=self.fake_url_1,
            )

        self.metric_2 = GraphicsMetric.objects.create(
            image=self.image,
            machine=self.machine,
            jenkins_build=self.jenkins_build_2,
            name='ps-intel-2500-le-x-gui-toolkit-QGears2-XRender ' +
                 '+ Image Scaling',
            ran_at=self.ran_at,
            minimum=self.min_10,
            maximum=self.max_100,
            average=self.avg_50,
            stddev=self.stddev_55,
            test_resolution='1024x768',
            framework=self.framework,
            jenkins_url=self.fake_url_0,
            )

        self.metric_3_xmir = GraphicsMetric.objects.create(
            image=self.image,
            machine=self.machine_xmir,
            jenkins_build=self.jenkins_build_2_xmir,
            name='ps-intel-2500-le-xmir-gui-toolkit-QGears2-XRender ' +
                 '+ Image Scaling',
            ran_at=self.ran_at,
            minimum=self.min_10,
            maximum=self.max_100,
            average=self.avg_50,
            stddev=self.stddev_55,
            test_resolution='1024x768',
            framework=self.framework,
            jenkins_url=self.fake_url_1,
            )

        self.metric_3 = GraphicsMetric.objects.create(
            image=self.image,
            machine=self.machine,
            jenkins_build=self.jenkins_build_3,
            name='ps-intel-2500-le-x-gui-toolkit-QGears2-XRender ' +
                 '+ Image Scaling',
            ran_at=self.ran_at,
            minimum=self.min_10,
            maximum=self.max_100,
            average=self.avg_50,
            stddev=self.stddev_55,
            test_resolution='1024x768',
            framework=self.framework,
            jenkins_url=self.fake_url_1,
            )

        self.metric_3_xmir = GraphicsMetric.objects.create(
            image=self.image,
            machine=self.machine_xmir,
            jenkins_build=self.jenkins_build_3_xmir,
            name='ps-intel-2500-le-xmir-gui-toolkit-QGears2-XRender ' +
                 '+ Image Scaling',
            ran_at=self.ran_at,
            minimum=self.min_10,
            maximum=self.max_100,
            average=self.avg_50,
            stddev=self.stddev_55,
            test_resolution='1024x768',
            framework=self.framework,
            jenkins_url=self.fake_url_0,
            )

    def test_openarena_url(self):
        test_url = self.client.get(reverse('openarena',),)
        self.assertEqual(test_url.status_code, 200)
        self.assertTemplateUsed(test_url, 'graphics/openarena.html')

    def test_openarena_filter_card_type_url(self):
        test_url = self.client.get(
            reverse('openarena_filter_card_type', args=["intel"]),)
        self.assertEqual(test_url.status_code, 200)
        self.assertTemplateUsed(test_url, 'graphics/openarena.html')

    def test_openarena_filter_card_type_resolution_url(self):
        test_url = self.client.get(
            reverse('openarena_filter_card_type_resolution',
                    args=["intel", "800x600"]),)
        self.assertEqual(test_url.status_code, 200)
        self.assertTemplateUsed(test_url, 'graphics/openarena.html')

    def test_guitoolkits_url(self):
        test_url = self.client.get(reverse('guitoolkits',),)
        self.assertEqual(test_url.status_code, 200)
        self.assertTemplateUsed(test_url, 'graphics/guitoolkits.html')

    def test_guitoolkits_filter_card_type_url(self):
        test_url = self.client.get(
            reverse('guitoolkits_filter_card_type', args=["intel"]),)
        self.assertEqual(test_url.status_code, 200)
        self.assertTemplateUsed(test_url, 'graphics/guitoolkits.html')

    def test_guitoolkits_filter_card_type_resolution_url(self):
        test_url = self.client.get(
            reverse('guitoolkits_filter_card_type_resolution',
                    args=["intel", "800x600"]),)
        self.assertEqual(test_url.status_code, 200)
        self.assertTemplateUsed(test_url, 'graphics/guitoolkits.html')

    def test_nexuiz_url(self):
        test_url = self.client.get(reverse('nexuiz',),)
        self.assertEqual(test_url.status_code, 200)
        self.assertTemplateUsed(test_url, 'graphics/nexuiz.html')

    def test_nexuiz_filter_card_type_url(self):
        test_url = self.client.get(
            reverse('nexuiz_filter_card_type', args=["intel"]),)
        self.assertEqual(test_url.status_code, 200)
        self.assertTemplateUsed(test_url, 'graphics/nexuiz.html')

    def test_nexuiz_filter_card_type_resolution_url(self):
        test_url = self.client.get(
            reverse('nexuiz_filter_card_type_resolution',
                    args=["intel", "800x600"]),)
        self.assertEqual(test_url.status_code, 200)
        self.assertTemplateUsed(test_url, 'graphics/nexuiz.html')

    def test_api_graphics_api_latest_filter_resolution_url_no_guitoolkit(self):
        test_url = self.client.get(
            reverse('graphics_api_latest_filter_resolution',
                    args=["openarena", "intel", "800x600"]),)
        self.assertEqual(test_url.status_code, 200)

        content = json.loads(test_url.content)
        self.assertEqual(content[1]['key'], 'ps-intel-2500-le-x-openarena')
        self.assertEqual(content[1]['values'][0][0], self.iso)
        self.assertEqual(content[1]['values'][0][1], self.avg_5)
        self.assertEqual(content[1]['values'][0][2], '800x600')
        self.assertEqual(content[1]['values'][0][3], self.max_10)
        self.assertEqual(content[1]['values'][0][4], self.min_1)
        self.assertEqual(content[1]['values'][0][5], self.stddev_4)
        self.assertEqual(content[1]['values'][0][6], self.fake_url_0)

    def test_api_graphics_api_latest_filter_resolution_url_guitoolkits(self):
        """
        graphic_api_latest_filter_resolution must return json content

        """
        test_url = self.client.get(
            reverse('graphics_api_latest_filter_resolution',
                    args=["gui-toolkit", "intel", "1024x768"]),)
        self.assertEqual(test_url.status_code, 200)
        content = json.loads(test_url.content)
        self.assertEqual(content[0]['key'],
                         'ps-intel-2500-le-x-gui-toolkit-QGears2-XRender ' +
                         '+ Image Scaling')
        self.assertEqual(content[0]['values'][0][0], self.iso)
        self.assertEqual(content[0]['values'][0][1], self.avg_50)
        self.assertEqual(content[0]['values'][0][2], '1024x768')
        self.assertEqual(content[0]['values'][0][3], self.max_100)
        self.assertEqual(content[0]['values'][0][4], self.min_10)
        self.assertEqual(content[0]['values'][0][5], self.stddev_55)
        self.assertEqual(content[0]['values'][0][6], self.fake_url_0)

    def test_api_graphics_api_result_filter_resolution_url(self):
        """
        test graphics_api_result_filter_resolution
        must not filter job name, the key name must match
        (unlike latest filter)

        """
        test_url = self.client.get(
            reverse('graphics_api_result_filter_resolution',
                    args=["gui-toolkit", "intel", "1024x768"]),)
        self.assertEqual(test_url.status_code, 200)
        content = json.loads(test_url.content)
        self.assertEqual(content[1]['key'],
                         'ps-intel-2500-le-xmir-gui-toolkit-QGears2-XRender' +
                         ' + Image Scaling')
        self.assertEqual(content[1]['values'][0][0], format(self.ran_at, 'U'))
        self.assertEqual(content[1]['values'][0][1], self.avg_50)
        self.assertEqual(content[1]['values'][0][2], '1024x768')
        self.assertEqual(content[1]['values'][0][3], self.max_100)
        self.assertEqual(content[1]['values'][0][4], self.min_10)
        self.assertEqual(content[1]['values'][0][5], self.stddev_55)
        self.assertEqual(content[1]['values'][0][6], self.fake_url_1)


class TestJenkinsPullPhoronix(TestCase):
    def setUp(self):
        self.url_list = ["https://jenkins.qa.ubuntu.com/job/" +
                         "openarena-benchmark-ps-radeon-hd5750-le/56/" +
                         "artifact/results/benchmark_data.json"]

        localdir = path.dirname(__file__)
        with open(path.join(localdir, 'data/jenkins_get.json'), 'r') as f:
            jenkins_get_data = f.read()
        f.close()
        self.jdata = jenkins_get_data

    def test_jenkins_get_from_common_exists(self):
        """
        jenkins_get function must be in common.management
        """
        from common.management import jenkins_get

    def test_jenkins_pull_phoronix_add_to_database(self):
        """
        jenkins_pull_phoronix must add a job to a database
        """
        #mock url_list
        jenkins_pull.url_artifact_list = Mock(return_value=self.url_list)
        #mock jenkins_get
        management.jenkins_get = Mock(return_value=json.loads(self.jdata))
        #jenkins_pull_phoronix with mocked url_artifact_list and jenkins_get
        jenkins_pull_phoronix.add_records(
            'openarena--benchmark-ps-radeon-hd5750-le')

        result = GraphicsMetric.objects.filter(
            name__contains='openarena',
            test_resolution='800x600',
            machine__name__contains='radeon',
            ).order_by('ran_at').latest('ran_at')

        self.assertEqual(result.maximum, 72.3)
        self.assertEquals(result.name, 'ps-radeon-hd5750-le-x-openarena')
        self.assertEquals(result.minimum, 72.3)
        self.assertEquals(result.average, 72.3)
        self.assertEquals(result.machine.name,
                          "ps-radeon-hd5750-le")
        self.assertEquals(result.image.build_number, "20130905")
        self.assertEquals(result.jenkins_url, self.url_list[0])
        self.assertEquals(result.stddev, 0.0)
        self.assertEquals(result.test_resolution, "800x600")
        self.assertEquals(result.framework, 'utah')
        self.assertEquals(result.image.release, "saucy")
