import jenkinsapi
from jenkinsapi import *
from qa_dashboard.settings import JENKINS_URL


def url_artifact_list(jenkins_job_name, artifact_name, last_build=0):
    """
    gets a specific artifact for all successful builds from
    specified job_name

    :param string jenkinks_job_name: the name of the job with the artifact
    :param string artifact_name: the file name of the artifact in jenkins
    :param integer last_build: the last build already stored in the database
    use this to return a list of only artifacts that are not in the database

    example:
    >>>jenkins_pull.url_list('openarena-ps-intel-2500-le', 'benchmark.json')

    return: List of urls
    """
    jenkapi = jenkinsapi.api.Jenkins(JENKINS_URL)
    job = jenkapi.get_job(jenkins_job_name)
    build_ids = job.get_build_ids()
    url_list = []
    for id in build_ids:
        if int(last_build) < int(id):
            build = job.get_build(id)
            #don't get artifacts from builds that have failed
            if build.get_status() is 'SUCCESS':
                url = [archive.url for archive in build.get_artifacts()
                       if artifact_name in archive.url]
                url_list.append(''.join(url))
    if url_list:
        return url_list
    else:
        return False
