from django.template import RequestContext
from django.views.decorators.http import require_GET
from django.shortcuts import render_to_response


help_info = str("""Double click circle to disable all other results.
                 Single click circle to enable/disable result.
                 Double click bar to see build results on jenkins.""")

resolutions = ["1024x768", "800x600"]

# add ps-radeon-hd8750-he, when it gets 3d acceleration
all_machines = {
    "intel": [
        "ps-intel-2500-le",
        "ps-intel-3000",
        "ps-intel-4000-he",
        ],
    "nvidia": [
        "ps-nvidia-310",
        "ps-nvidia-gt440-le",
        "ps-nvidia-gt640-he",
        ],
    "radeon": [
        "ps-radeon-hd5750-le",
        "ps-radeon-hd7450-le"
        ],
    }


@require_GET
def openarena(request,
              card_type_filter="ps",
              resolution_filter="1024x768"):

    if card_type_filter in all_machines:
        oa_machines = {card_type_filter: all_machines[card_type_filter]}
    else:
        oa_machines = {
            "intel": [
                "ps-intel-2500-le",
                "ps-intel-3000",
                "ps-intel-4000-he",
            ],
            "nvidia": [
                "ps-nvidia-310",
            ],
            "radeon": [
                "ps-radeon-hd5750-le",
                "ps-radeon-hd7450-le"
            ],
        }

    data = {'help_info': help_info,
            'machines': oa_machines,
            'all_machines': all_machines,
            'resolution_filter': resolution_filter,
            'card_type_filter': card_type_filter,
            'resolutions': resolutions,
            }

    return render_to_response('graphics/openarena.html',
                              data,
                              RequestContext(request))


@require_GET
def gui_toolkits(request,
                 card_type_filter="ps",
                 resolution_filter="1024x768"):

    if card_type_filter in all_machines:
        gt_machines = {card_type_filter: all_machines[card_type_filter]}
    else:
        gt_machines = {
            "intel": [
                "ps-intel-2500-le",
                "ps-intel-3000",
                "ps-intel-4000-he",
            ],
            "nvidia": [
                "ps-nvidia-310",
            ],
            "radeon": [
                "ps-radeon-hd5750-le",
                "ps-radeon-hd7450-le"
            ],
        }

    data = {'help_info': help_info,
            'machines': gt_machines,
            'all_machines': all_machines,
            'resolution_filter': resolution_filter,
            'card_type_filter': card_type_filter,
            'resolutions': resolutions,
            }

    return render_to_response('graphics/guitoolkits.html',
                              data,
                              RequestContext(request))


@require_GET
def nexuiz(request,
           card_type_filter="ps",
           resolution_filter="1024x768"):

    if card_type_filter in all_machines:
        n_machines = {card_type_filter: all_machines[card_type_filter]}
    else:
        n_machines = {
            "intel": [
                "ps-intel-2500-le",
                "ps-intel-3000",
                "ps-intel-4000-he",
            ],
        }

    data = {'help_info': help_info,
            'machines': n_machines,
            'all_machines': all_machines,
            'resolution_filter': resolution_filter,
            'card_type_filter': card_type_filter,
            'resolutions': resolutions,
            }

    return render_to_response('graphics/nexuiz.html',
                              data,
                              RequestContext(request))
